==> sife-0.0.37.tgz: package/lib/themeBuilder.js <==
var fs = require('fs'),
	path = require('path'),
	componentBuilder = require('./componentBuilder.js'),
	_ = require('underscore');

module.exports = {

	init: function(conf) {
		this.conf = conf;
		this.componentBuilder = componentBuilder;
		this.componentBuilder.init(conf);
	},

	createTasks: function(gruntHelper, taskName, taskList){
		var themes = this.getThemesFromFiles(this.conf.sources.themes),
			files,themeName,component,tmpPath,jsComponents,themeFilePath,
			oriTaskName = taskName;

		for (var i = 0; i < themes.length; i++) {
			themeName = themes[i].name;
			taskName = oriTaskName + '_'+themeName;
			themeFilePath = path.join(this.conf.output.css.themes, themeName + '.css');
			jsComponents = '';

			// variables + theme vars + theme-base
			files = [
				path.join(this.conf.sources.components, 'variables.css'),
				path.join(this.conf.sources.themes, themeName, 'css', 'variables.css'),
				path.join(this.conf.output.css.base, 'theme-base.css'),

			];

			// theme dependencies
			if(themes[i].dependencies){
				for (var ii = 0; ii < themes[i].dependencies.length; ii++) {
					component = this.componentBuilder.getComponentById(themes[i].dependencies[ii]);
					files = files.concat(this.componentBuilder.getComponentCssDependencyFiles(component, true));

					if(component.dependencies.js){
						console.log('theme '+themeName+' need '+component.id);
						jsComponents+= 'jsComponents[\''+component.id+'\'] = require(\''+path.resolve(component.path)+'/js/'+component.name+'.js\');\n';
					}
				}
			}

			// theme main.js
			var themeJSPath = path.join(this.conf.sources.themes, themeName, 'js', 'main.js');
			if(fs.lstatSync(themeJSPath).isFile()){
				jsComponents+= 'jsComponents[\'main\'] = require(\''+path.resolve(themeJSPath)+'\')';
			}

			// and finally the theme skin
			files.push(path.join(this.conf.sources.themes, themeName, 'css', 'skin.css'));
			taskList.push('concat:'+taskName);

			gruntHelper.concat.add(taskName, themeFilePath, files);

			// and extract rules vars
			var extractConfig = {};
			extractConfig[themeName] = {src:[themeFilePath]};
			gruntHelper.setGruntConfig('extractrulesvars', extractConfig);
			taskList.push('extractrulesvars:'+themeName);

			// JST
			if(themes[i].JST){
				var componentFiles = [];
				for (var ii = 0; ii < themes[i].JST.length; ii++) {
					component = this.componentBuilder.getComponentById(themes[i].JST[ii]);
					componentFiles.push(component.path+'/html/*.hbs');
				}

				gruntHelper.handlebars.add('theme_'+themeName, this.conf.output.jst + '/theme_'+themeName+'.js', componentFiles, {partialsPathRegex: /.*/,components:true});
				taskList.push('handlebars:theme_'+themeName);
			}


			if(jsComponents !== ''){
				var jsCode = fs.readFileSync(path.resolve(__dirname,'../srcs/App.js')).toString();
				jsCode = jsCode.replace('/* __INSERT_JS_COMPONENTS__ */', jsComponents);
				jsCode = jsCode.replace('/* __INSERT_BREAKPOINTS__ */', 'App.breakpoints = '+JSON.stringify(this.conf.build.grid.breakpoints)+';');
				jsCode+= '\n';
				//jsCode = 'window.App = {components:{},breakpoints:'+JSON.stringify(this.conf.build.grid.breakpoints)+',"@media":function mq(i){return i=i.replace("--",""),"matchMedia"in window?window.matchMedia("(min-width: "+App.breakpoints[i]+"em)").matches:!1}};\n'
				//+ 'var sifeComponents = {};\n' + jsCode + '\n$("[data-sife-js]").each(function(){function e(){n++,n===t.externalSources.length&&t($(this))}var s=$(this).attr("data-sife-js");if(sifeComponents[s]){var t=sifeComponents[s],n=0;if(t.externalSources)for(var r=0;r<t.externalSources.length;r++)t.externalSources[r].indexOf(".css")>-1?loadCSS(t.externalSources[r],null,e.bind(this)):$.getScript(t.externalSources[r],e.bind(this));else t($(this))}});\n';

				if(themes[i].externalDependencies){
					_.each(themes[i].externalDependencies, function(depPath, depKey){
						jsCode+= depKey+' = require(\''+path.resolve(process.env.PWD,depPath)+'\')\n';
					});

				}

				tmpPath = '/tmp/sife_themjs_'+themeName+'.js';
				fs.writeFileSync(tmpPath, jsCode);

				gruntHelper.copy.add(taskName, this.conf.output.js+'/themes/'+themeName+'.js', [tmpPath]);
				taskList.push('copy:'+taskName);

				if(themes[i].JST){
					gruntHelper.concat.add(taskName+'join', this.conf.output.js+'/themes/'+themeName+'.js', [
						this.conf.output.jst + '/theme_'+themeName+'.js',
						this.conf.output.js+'/themes/'+themeName+'.js'
					]);
					taskList.push('concat:'+taskName+'join');
				}

				gruntHelper.browserify.add(taskName, this.conf.output.js+'/themes/'+themeName+'-bundle.js', this.conf.output.js+'/themes/'+themeName+'.js');
				taskList.push('browserify:'+taskName);
			}

		}
	},

	getThemesFromFiles: function(themesPath){
		var dirs = fs.readdirSync(themesPath),
			themes = [],
			theme,themeConfig;
		for (var i = 0; i < dirs.length; i++) {
			themeConfig = this.getThemeConfig(dirs[i]);
			theme = {
				name: dirs[i],
				dependencies: themeConfig.dependencies,
				JST: themeConfig.JST,
				externalDependencies: themeConfig.externalDependencies
			};
			themes.push(theme);
		}
		return themes;
	},

	getThemeConfig: function(themeName){
		var config = {};
		try{
			config = JSON.parse(fs.readFileSync(path.join(this.conf.sources.themes, themeName, 'config.json')));
		}catch(err){
			config = {};
		}
		return config;
	}
};

==> sife-0.0.42.tgz: package/lib/themeBuilder.js <==
var fs = require('fs'),
	path = require('path'),
	componentBuilder = require('./componentBuilder.js'),
	_ = require('underscore');

module.exports = {

	init: function(conf) {
		this.conf = conf;
		this.componentBuilder = componentBuilder;
		this.componentBuilder.init(conf);
	},

	createTasks: function(gruntHelper, taskName, taskList){
		var themes = this.getThemesFromFiles(this.conf.sources.themes),
			files,themeName,component,tmpPath,jsComponents,themeFilePath,
			oriTaskName = taskName;

		for (var i = 0; i < themes.length; i++) {
			themeName = themes[i].name;
			taskName = oriTaskName + '_'+themeName;
			themeFilePath = path.join(this.conf.output.css.themes, themeName + '.css');
			jsComponents = '';

			// variables + theme vars + theme-base
			files = [
				path.join(this.conf.sources.components, this.conf.variablesFile || 'variables.css'),
				path.join(this.conf.sources.themes, themeName, 'css', this.conf.variablesFile || 'variables.css'),
				path.join(this.conf.output.css.base, 'theme-base.css'),

			];

			// theme dependencies
			if(themes[i].dependencies){
				for (var ii = 0; ii < themes[i].dependencies.length; ii++) {
					component = this.componentBuilder.getComponentById(themes[i].dependencies[ii]);
					files = files.concat(this.componentBuilder.getComponentCssDependencyFiles(component, true));

					if(component.dependencies.js){
						console.log('theme '+themeName+' need '+component.id);
						jsComponents+= 'jsComponents[\''+component.id+'\'] = require(\''+path.resolve(component.path)+'/js/'+component.name+'.js\');\n';
					}
				}
			}

			// theme main.js
			var themeJSPath = path.join(this.conf.sources.themes, themeName, 'js', 'main.js');
			if(fs.lstatSync(themeJSPath).isFile()){
				jsComponents+= 'jsComponents[\'main\'] = require(\''+path.resolve(themeJSPath)+'\')';
			}

			// and finally the theme skin
			files.push(path.join(this.conf.sources.themes, themeName, 'css', 'skin.css'));
			taskList.push('concat:'+taskName);

			gruntHelper.concat.add(taskName, themeFilePath, files);

			// and extract rules vars
			var extractConfig = {};
			extractConfig[themeName] = {src:[themeFilePath]};
			gruntHelper.setGruntConfig('extractrulesvars', extractConfig);
			taskList.push('extractrulesvars:'+themeName);

			// JST
			if(themes[i].JST){
				var componentFiles = [];
				for (var ii = 0; ii < themes[i].JST.length; ii++) {
					component = this.componentBuilder.getComponentById(themes[i].JST[ii]);
					componentFiles.push(component.path+'/html/*.hbs');
				}

				gruntHelper.handlebars.add('theme_'+themeName, this.conf.output.jst + '/theme_'+themeName+'.js', componentFiles, {partialsPathRegex: /.*/,components:true});
				taskList.push('handlebars:theme_'+themeName);
			}


			if(jsComponents !== ''){
				var jsCode = fs.readFileSync(path.resolve(__dirname,'../srcs/App.js')).toString();
				jsCode = jsCode.replace('/* __INSERT_JS_COMPONENTS__ */', jsComponents);
				jsCode = jsCode.replace('/* __INSERT_BREAKPOINTS__ */', 'App.breakpoints = '+JSON.stringify(this.conf.build.grid.breakpoints)+';');
				jsCode+= '\n';
				//jsCode = 'window.App = {components:{},breakpoints:'+JSON.stringify(this.conf.build.grid.breakpoints)+',"@media":function mq(i){return i=i.replace("--",""),"matchMedia"in window?window.matchMedia("(min-width: "+App.breakpoints[i]+"em)").matches:!1}};\n'
				//+ 'var sifeComponents = {};\n' + jsCode + '\n$("[data-sife-js]").each(function(){function e(){n++,n===t.externalSources.length&&t($(this))}var s=$(this).attr("data-sife-js");if(sifeComponents[s]){var t=sifeComponents[s],n=0;if(t.externalSources)for(var r=0;r<t.externalSources.length;r++)t.externalSources[r].indexOf(".css")>-1?loadCSS(t.externalSources[r],null,e.bind(this)):$.getScript(t.externalSources[r],e.bind(this));else t($(this))}});\n';

				if(themes[i].externalDependencies){
					_.each(themes[i].externalDependencies, function(depPath, depKey){
						jsCode+= depKey+' = require(\''+path.resolve(process.env.PWD,depPath)+'\')\n';
					});

				}

				tmpPath = '/tmp/sife_themjs_'+themeName+'.js';
				fs.writeFileSync(tmpPath, jsCode);

				gruntHelper.copy.add(taskName, this.conf.output.js+'/themes/'+themeName+'.js', [tmpPath]);
				taskList.push('copy:'+taskName);

				if(themes[i].JST){
					gruntHelper.concat.add(taskName+'join', this.conf.output.js+'/themes/'+themeName+'.js', [
						this.conf.output.jst + '/theme_'+themeName+'.js',
						this.conf.output.js+'/themes/'+themeName+'.js'
					]);
					taskList.push('concat:'+taskName+'join');
				}

				gruntHelper.browserify.add(taskName, this.conf.output.js+'/themes/'+themeName+'-bundle.js', this.conf.output.js+'/themes/'+themeName+'.js');
				taskList.push('browserify:'+taskName);
			}

		}
	},

	getThemesFromFiles: function(themesPath){
		var dirs = fs.readdirSync(themesPath),
			themes = [],
			theme,themeConfig;


		for (var i = 0; i < dirs.length; i++) {

			if(fs.lstatSync(path.join(themesPath,dirs[i])).isDirectory()){

				themeConfig = this.getThemeConfig(dirs[i]);
				theme = {
					name: dirs[i],
					dependencies: themeConfig.dependencies,
					JST: themeConfig.JST,
					externalDependencies: themeConfig.externalDependencies
				};
				themes.push(theme);
			}

		}
		return themes;
	},

	getThemeConfig: function(themeName){
		var config = {};
		var foundConfig = true;
		try{
			config = JSON.parse(fs.readFileSync(path.join(this.conf.sources.themes, themeName, 'config.json')));
		}catch(err){
			foundConfig = false;
			config = {};
		}
		if(!foundConfig){
			try{
				config = require(path.join(process.cwd(), this.conf.sources.themes, themeName, 'config.js'));
			}catch(err){
				config = {};
			}
		}

		return config;
	}
};
